using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;

using VirtualLab.Programming;
using VirtualLabAPI.Core.BasicFunctions;
using VirtualLabAPI.Core.Common;
using VirtualLabAPI.Core.DataVisualization;
using VirtualLabAPI.Core.FieldRepresentations;
using VirtualLabAPI.Core.Functions;
using VirtualLabAPI.Core.GeometryDescription;
using VirtualLabAPI.Core.LightPath;
using VirtualLabAPI.Core.Materials;
using VirtualLabAPI.Core.Modules;
using VirtualLabAPI.Core.Numerics;
using VirtualLabAPI.Core.Numerics.Region2D;
using VirtualLabAPI.Core.OpticalSystems;
using VirtualLabAPI.Core.Propagation;
using VirtualLabAPI.Core.StandardTransmissionDesign;
using VirtualLabAPI.Core.Threading;

namespace OwnCode {
    public class VLModule : IVLModule {
        //the path where all the data is located; must be specified by the user
        string pathofIFTAInputData = @"C:\...\";
        //file name of the IFTA document which should be loaded from hard disc
        string filenameIFTA = "Iterative Fourier Transform Algorithm Optimization.dp";
        //define filename for storage of merit function values
        string filenameMeritFunctionValues = "Result.txt";
                     
        public void Run() {
            //load IFTA from hard disc
            DesignAlgorithmHandler design = DesignAlgorithmHandler.Load(Path.Combine(pathofIFTAInputData, filenameIFTA));
            //error handling
            if(design == null){
                Globals.DataDisplay.LogError("IFTA could not be loaded!");
                return;
            }                 
                     
            //read sampling parameters from design document
            SamplingParameters sPara = new SamplingParameters();
            sPara = new SamplingParameters(design.ConstraintSpecification.SamplingPoints,
                                           design.ConstraintSpecification.SamplingDistance);
            
            if(design.Transmission != null){
                //show initial transmission
                ComplexAmplitude initialTransmission = new ComplexAmplitude(design.Transmission);
                initialTransmission.SamplingDistance = sPara.SamplingDistance;
                Globals.DataDisplay.ShowDocument(initialTransmission, "Initial Transmission");
            }
            
            //perform design
            IterationDataOutput iterationDataOutput = new IterationDataOutput();
            WorkerThread worker = new WorkerThread("Design", null, null, null);
            design.DoAllSelectedSteps(iterationDataOutput, worker);
            
            //show optimized transmission
            ComplexAmplitude optimizedTransmission = new ComplexAmplitude(design.Transmission);
            optimizedTransmission.SamplingDistance = sPara.SamplingDistance;
            Globals.DataDisplay.ShowDocument(optimizedTransmission, "Optimized Transmission");
            
            SamplingParameters sParaOutPutField = new SamplingParameters();
            sParaOutPutField = new SamplingParameters(design.ConstraintSpecification.SamplingPoints,
                                                      design.ConstraintSpecification.SamplingDistanceOut);
            
            //calculate performance of designed phase
            ComplexField cfOutputField = design.DoForwardProp();
            double conversionEff = design.GetConversionEfficiency(cfOutputField, sParaOutPutField);
            double uniformityError = design.GetUniformityError(cfOutputField, sParaOutPutField);
            
            //log performance of design
            Globals.DataDisplay.LogMessage("Conversion Efficiency = " + PhysicalUnits.FormatPhysicalUnit(conversionEff, PhysicalProperty.Percentage));
            Globals.DataDisplay.LogMessage("Uniformity Error = " + PhysicalUnits.FormatPhysicalUnit(uniformityError, PhysicalProperty.Percentage));    
      
            //define path for storing performance values
            string filenameMeritFunction = Path.Combine(pathofIFTAInputData, filenameMeritFunctionValues);   
            //check whether file already exist
            if(File.Exists(filenameMeritFunction)){
                File.Delete(filenameMeritFunction);
            }
            
            //write performance values to text file
            VL_Files.WriteLineToLogFile(filenameMeritFunction, "Conversion Efficiency = " + PhysicalUnits.FormatPhysicalUnit(conversionEff, PhysicalProperty.Percentage));
            VL_Files.WriteLineToLogFile(filenameMeritFunction, "Uniformity Error = " + PhysicalUnits.FormatPhysicalUnit(uniformityError, PhysicalProperty.Percentage));
        }
    }
}